import importlib
import os
import sys
from typing import Any, Dict, List, Optional, Union

from pydantic import AnyHttpUrl, BaseSettings, HttpUrl, validator


class Settings(BaseSettings):
    SECRET_KEY: str
    ALGORITHM: str = "HS256"
    AUTH_USER_MODEL_NAME: str = "User"
    AUTH_APP_NAME: str = "fast_tmp"
    ACCESS_TOKEN_EXPIRE_MINUTES: int = 60 * 24 * 8
    BACKEND_CORS_ORIGINS: List[AnyHttpUrl] = []
    FASTAPI_SETTINGS_MODULE: Optional[str]
    DATABASE_URL: str
    DEBUG: bool = True
    LOGIN_URL: str = "/api-token-auth"

    @validator("DEBUG", pre=True, allow_reuse=True)
    def get_debug(cls, v: str) -> bool:
        if isinstance(v, str):
            if v != "True":
                return False
        return True

    @validator("BACKEND_CORS_ORIGINS", pre=True, allow_reuse=True)
    def assemble_cors_origins(cls, v: Union[str, List[str]]) -> Union[List[str], str]:
        if isinstance(v, str) and not v.startswith("["):
            return [i.strip() for i in v.split(",")]
        elif isinstance(v, (list, str)):
            return v
        raise ValueError(v)

    SENTRY_DSN: Optional[HttpUrl] = None
    SENTRY_ENVIROMENT: str = "development"

    @validator("SENTRY_DSN", pre=True, allow_reuse=True)
    def sentry_dsn_can_be_blank(cls, v: str) -> Optional[str]:
        if v and len(v) > 0:
            return v
        return None

    DB_TYPE: str = "mysql"
    # 额外的配置信息
    EXTRA_SETTINGS: Dict[str, Any] = {}

    # cas相关配置
    CAS_SERVER_URL: str = "/cas/"

    class Config:
        case_sensitive = True
        env_file = ".env"

    def __init__(self):
        super(Settings, self).__init__()

        workdir = os.getcwd()  # 把工作路径加入到代码执行里面
        for path in sys.path:
            if path == workdir:
                break
        else:
            sys.path.append(workdir)
        if self.FASTAPI_SETTINGS_MODULE:
            try:
                mod = importlib.import_module(self.FASTAPI_SETTINGS_MODULE)
                for setting in dir(mod):
                    if setting.isupper():
                        setting_value = getattr(mod, setting)
                        if hasattr(self, setting):
                            setattr(self, setting, setting_value)
                        else:
                            self.EXTRA_SETTINGS[setting] = setting_value
            except Exception as e:
                raise ImportError(f"导入settings报错:{e}")

        if self.SENTRY_DSN:
            import sentry_sdk

            sentry_sdk.init(
                dsn=self.SENTRY_DSN,
                environment=self.SENTRY_ENVIROMENT,
            )


settings = Settings()



from typing import Optional, Union

from pydantic import HttpUrl
from pydantic.main import BaseModel

from .abstract_schema import _Action
from .enums import ActionTypeEnum, ButtonLevelEnum, TypeEnum
from .frame import Dialog, Drawer


class FeedBack(BaseModel):  # 和弹窗一致
    title: str
    body: Union[str, BaseModel]


class ActionMessage(BaseModel):
    success: str = "操作成功"
    failed: str = "操作失败"


# 参考：https://baidu.gitee.io/amis/docs/components/action?page=1
class AjaxAction(_Action):
    """
    ajax请求
    """

    actionType = ActionTypeEnum.ajax
    confirmText: Optional[str] = None  # 如果配置了这个字段则会有弹出框提示
    api: str
    redirect: Optional[str] = None  # 如果配置路径，可以实现跳转
    feedback: Optional[FeedBack] = None
    # 如果重载需要携带参数，则可以输入：{"reload": "xxx?a=${a}&b=${b}"}
    reload: Optional[str] = None  # 可以把name传递过来，则请求成功自动刷新,页面刷新则为window，多个用逗号分开
    message: Optional[ActionMessage] = None  # 修改默认toast信息


# class LinkAction(_Action):
#     """
#     单页跳转(暂不支持)
#     """
#     actionType = ActionTypeEnum.link
#     level = ButtonLevelEnum.info
class UrlAction(_Action):
    """
    单页跳转(暂不支持)
    """

    actionType = ActionTypeEnum.url
    level = ButtonLevelEnum.success
    url: HttpUrl
    blank: bool = False


class CopyAction(_Action):
    """
    复制文本
    """

    actionType = ActionTypeEnum.copy
    content: str  # 制定要复制的内容


# class ReloadAction(_Action):
#     type = ActionTypeEnum.reload
#     target: str  # 需要刷新的目标组件名字（组件的name值，自己配置的），多个请用 , 号隔开。


class DialogAction(_Action):
    type = TypeEnum.button
    actionType = ActionTypeEnum.dialog
    dialog: Dialog


class DrawerAction(_Action):
    actionType = ActionTypeEnum.drawer
    drawer: Drawer
    # type = TypeEnum.button



from enum import Enum


class FormWidgetSize(
    str,
    Enum,
):
    full = "full"
    lg = "lg"  # 大
    md = "md"  # 中,默认值
    sm = "sm"  # 小
    xs = "xs"  # 极小


class ControlEnum(str, Enum):
    text = "text"
    textarea = "textarea"
    input_text = "input-text"
    switch = "switch"  # 开关
    select = "select"
    number = "input-number"
    native_number = "native-number"  # exploer natave number
    array = "array"
    date = "date"
    datetime = "input-datetime"
    uuid = "uuid"
    rich_text = "rich-text"
    time = "time"
    mapping = "mapping"
    transfer = "transfer"
    checkboxes = "checkboxes"
    picker = "picker"


class ItemModel(str, Enum):
    horizontal = "horizontal"
    inline = "inline"
    normal = "normal"


class WidgetSize:
    lg = "lg"  # 大
    md = "md"  # 中,默认值
    sm = "sm"  # 小
    xs = "xs"  # 极小


class ValidateEnum(str, Enum):
    isEmptyString = "isEmptyString"  # 必须是空白字符。注意！ 该格式校验是值，校验空白字符，而不是当前表单项是否为空，想校验是否为空，请配置
    isEmail = "isEmail"  # 必须是Email。
    isUrl = "isUrl"  # 必须是Url。
    isNumeric = "isNumeric"  # 必须是数值。
    isAlpha = "isAlpha"  # 必须是字母。
    isAlphanumeric = "isAlphanumeric"  # 必须是字母或者数字。
    isInt = "isInt"  # 必须是整形。
    isFloat = "isFloat"  # 必须是浮点形。
    isLength = "isLength"  #: length 是否长度正好等于设定值。
    minLength = "minLength"  #: length 最小长度。
    maxLength = "maxLength"  #: length最大长度。
    maximum = "maximum"  # number    最大值。
    minimum = "minimum"  # number    最小值。
    equals = "equals"  #: xxx    当前值必须完全等于    xxx。
    equalsField = "equalsField"  #: xxx    当前值必须与    xxx    变量值一致。
    isJson = "isJson"  # 是否是合法的Json字符串。
    notEmptyString = "notEmptyString"  # 要求输入内容不是空白。
    isUrlPath = "isUrlPath"  # 是    url    路径。
    matchRegexp = "matchRegexp"  #: / foo / 必须命中某个正则。
    matchRegexp1 = "matchRegexp1"  #: / foo / 必须命中某个正则。
    matchRegexp2 = "matchRegexp2"  #: / foo / 必须命中某个正则。
    matchRegexp3 = "matchRegexp3"  #: / foo / 必须命中某个正则。
    matchRegexp4 = "matchRegexp4"  #: / foo / 必须命中某个正则。



import binascii
import hashlib
from typing import Optional

from fast_tmp.conf import settings


def make_password(raw_password: str) -> str:
    """
    加密密码,返回加密的密码值和随机盐
    """
    password = hashlib.pbkdf2_hmac(
        "sha256", raw_password.encode("utf-8"), settings.SECRET_KEY.encode("utf-8"), 16
    )  # 随机生成盐值
    return binascii.hexlify(password).decode()


def verify_password(raw_password: str, password: Optional[str]) -> bool:
    """
    验证密码是否正确
    :param raw_password:要验证的密码
    :param password:数据库存储的密码
    """
    if not password or not raw_password:
        return False
    random_salt = settings.SECRET_KEY.encode("utf-8")
    raw_password_bytes = hashlib.pbkdf2_hmac(
        "sha256", raw_password.encode("utf-8"), random_salt, 16
    )
    if binascii.hexlify(raw_password_bytes).decode() == password:
        return True
    else:
        return False



# from fastapi import FastAPI
# from fastapi.openapi.docs import (
#     get_redoc_html,
#     get_swagger_ui_html,
#     get_swagger_ui_oauth2_redirect_html,
# )
# from fastapi.staticfiles import StaticFiles
#
#  todo 增加离线环境的openapi文件注册
# def offline_swagger(app: FastAPI):
#     """
#     离线环境下使用swagger,需要的静态文件可以执行 fast-tmp staticfile
#     """
#     assert app.redoc_url is None
#     assert app.docs_url is None
#     app.mount("/static", StaticFiles(directory="static"), name="static")
#
#     @app.get("/docs", include_in_schema=False)
#     async def custom_swagger_ui_html():
#         return get_swagger_ui_html(
#             openapi_url=app.openapi_url,
#             title=app.title + " - Swagger UI",
#             oauth2_redirect_url=app.swagger_ui_oauth2_redirect_url,
#             swagger_js_url="/static/swagger-ui-bundle.js",
#             swagger_css_url="/static/swagger-ui.css",
#         )
#
#     @app.get(app.swagger_ui_oauth2_redirect_url, include_in_schema=False)
#     async def swagger_ui_redirect():
#         return get_swagger_ui_oauth2_redirect_html()
#
#     @app.get("/redoc", include_in_schema=False)
#     async def redoc_html():
#         return get_redoc_html(
#             openapi_url=app.openapi_url,
#             title=app.title + " - ReDoc",
#             redoc_js_url="/static/redoc.standalone.js",
#         )



from typing import Optional, TypeVar

from pydantic.main import BaseModel

from fast_tmp.admin.schema.enums import ActionTypeEnum, ButtonLevelEnum, ButtonSize, TypeEnum


class BaseAmisModel(BaseModel):
    """
    所有amis类的父类
    """

    type: TypeEnum


AmisModel = TypeVar("AmisModel", bound=BaseAmisModel)


class _Action(BaseAmisModel):
    """
    操作按钮
    """

    type = TypeEnum.action
    label: str
    actionType: ActionTypeEnum
    icon: Optional[str]
    size: ButtonSize = ButtonSize.md
    level: ButtonLevelEnum = ButtonLevelEnum.primary
    tooltip: Optional[str]  # 鼠标挪上去的提示


class Message(BaseModel):
    """
    消息框
    """

    fetchSucss: Optional[str]
    fetchFailed: Optional[str]
    saveSuccess: Optional[str]
    saveFailed: Optional[str]



from fast_tmp.conf import settings  # 放第一个不要变
from starlette.applications import Starlette
# from starlette.middleware.sessions import SessionMiddleware
from tortoise.contrib.fastapi import register_tortoise
from starlette.middleware.cors import CORSMiddleware
from fastapi import FastAPI

from fast_tmp.depends.auth import register_app

settings._init_model()


def init_app(main_app: Starlette):
    @main_app.on_event("startup")
    async def startup() -> None:
        pass
        # await AsyncRedisUtil.init(**settings.REDIS)

    @main_app.on_event("shutdown")
    async def shutdown() -> None:
        pass
        # await AsyncRedisUtil.close()


def create_app() -> FastAPI:
    app = FastAPI(title='{{cookiecutter.project_slug}}', debug=settings.DEBUG)
    # 注册app的位置
    register_app(app)
    from {{cookiecutter.project_slug}}.apps.api.v1 import router
    app.include_router(router)
    if settings.BACKEND_CORS_ORIGINS:
        app.add_middleware(
            CORSMiddleware,
            allow_origins=[str(origin) for origin in
                           settings.BACKEND_CORS_ORIGINS],
            allow_credentials=True,
            allow_methods=["*"],
            allow_headers=["*"],
        )
    if settings.SENTRY_DSN:
        import sentry_sdk
        sentry_sdk.init(
            dsn=settings.SENTRY_DSN,
            environment=settings.SENTRY_ENVIROMENT,
        )
        # Sentry的插件
        from sentry_sdk.integrations.asgi import SentryAsgiMiddleware
        app.add_middleware(SentryAsgiMiddleware)
    # app.middleware("http")(cas_middleware)
    # 在cookie存储session
    # app.add_middleware(SessionMiddleware, secret_key=settings.CAS_SESSION_SECRET)
    register_tortoise(app, config=settings.TORTOISE_ORM)
    init_app(app)
    return app



import importlib
from typing import Any


def import_module(module_path: str) -> Any:
    path_list = module_path.split(".")
    mod = importlib.import_module(".".join(path_list[0:-1]))
    for k, v in mod.__dict__.items():
        if k == path_list[-1]:
            return v



import datetime
import importlib
import os
import typer
import sys

app = typer.Typer()
for path in sys.path:
    if path == os.getcwd():
        break
else:
    sys.path.append(os.getcwd())
try:
    from fast_tmp.conf import settings
except Exception as e:
    print(f"warning:{e}")
    settings = None


@app.command()
def createsuperuser(username: str, password: str):
    """
    创建超级用户
    """
    import os
    project_slug = os.path.split(os.getcwd())[1]
    os.environ.setdefault('FASTAPI_SETTINGS_MODULE', project_slug + ".settings")
    from fast_tmp.models import User
    from fast_tmp.db import engine
    from sqlalchemy.orm import Session
    with Session(engine) as session:
        user = User(
            username=username, is_superuser=True)
        user.set_password(password)
        session.add(user)
        session.commit()
    print(f"创建{username}成功")


@app.command()
def startapp():
    """
    创建app
    """
    basedir = os.path.abspath(os.path.dirname(__file__))
    from cookiecutter.main import cookiecutter
    cookiecutter(basedir + "/tpl/app/")


@app.command()
def startproject():
    """
    创建项目
    """
    basedir = os.path.abspath(os.path.dirname(__file__))
    from cookiecutter.main import cookiecutter
    cookiecutter(basedir + "/tpl/project")


@app.command()
def staticfile():
    """
    离线环境下的swagger静态文件
    """
    basedir = os.path.abspath(os.path.dirname(__file__))
    from cookiecutter.main import cookiecutter
    cookiecutter(basedir + "/tpl/static")


# 导入自定义脚本执行方式
if settings and settings.EXTRA_SETTINGS.get("EXTRA_SCRIPT"):
    for i in settings.EXTRA_SETTINGS.get("EXTRA_SCRIPT"):
        path_list = i.split(".")
        mod = importlib.import_module(".".join(path_list[0:-1]))
        for k, v in mod.__dict__.items():
            if k == path_list[-1]:
                app.command()(v)


@app.command()
def access_token(username: str):
    """
    创建一个测试token
    """
    from fast_tmp.utils.token import create_access_token

    print(create_access_token(
        data={"sub": username},
        expires_delta=datetime.timedelta(
            hours=30
        )
    ))


def main():
    app()


if __name__ == "__main__":
    main()



from fastapi import FastAPI
from fast_tmp.conf import settings
from .routes.hello_fast_tmp import hello_fast_tmp_router
{{cookiecutter.app_name}}_app = FastAPI(
    title="{{cookiecutter.app_name}}",
    debug=settings.DEBUG,
)
{{cookiecutter.app_name}}_app.include_router(hello_fast_tmp_router)


import os

import pytest

from tortoise.contrib.test import finalizer, initializer


@pytest.fixture(scope="session", autouse=True)
def initialize_tests(request):
    db_url = os.environ.get("TORTOISE_TEST_DB", "sqlite://:memory:")
    initializer(
        ["fast_tmp.models","{{cookiecutter.project_slug}}.models"],
        db_url=db_url,
        app_label="fast_tmp",
    )
    request.addfinalizer(finalizer)



# -*- encoding: utf-8 -*-
"""
@File    : buttons.py
@Time    : 2021/1/2 17:13
@Author  : chise
@Email   : chise123@live.com
@Software: PyCharm
@info    :
"""
from typing import List

from pydantic import BaseModel

from fast_tmp.admin.schema.abstract_schema import _Action
from fast_tmp.admin.schema.enums import TypeEnum


class Operation(BaseModel):
    type = TypeEnum.operation
    label = "操作"
    buttons: List[_Action] = []



from typing import Generator

from sqlalchemy import create_engine
from sqlalchemy.orm import Session, sessionmaker

from fast_tmp.conf import settings

# from fast_tmp.models import Base

engine = create_engine(
    settings.DATABASE_URL,
    echo=settings.DEBUG,
    # connect_args={"check_same_thread": False},
)
SessionLocal = sessionmaker(
    autocommit=False,
    autoflush=False,
    future=True,
    bind=engine,
)


def get_db_session() -> Generator[Session, None, None]:
    with SessionLocal() as session:
        yield session


# Base.metadata.create_all(engine)



from typing import List, Union

from fast_tmp.admin.schema.abstract_schema import BaseAmisModel, _Action
from fast_tmp.admin.schema.buttons import Operation
from fast_tmp.admin.schema.enums import TypeEnum
from fast_tmp.admin.schema.forms import Column


class CRUD(BaseAmisModel):
    type = TypeEnum.crud
    api: str  # 相对路径
    # 可以在后面跟上按钮，则默认每一行都有按钮，
    # 参考：https://baidu.gitee.io/amis/docs/components/dialog?page=1
    columns: List[Union[Column, _Action, Operation]]
    affixHeader: bool = False

    class Config:
        arbitrary_types_allowed = True



from datetime import datetime, timedelta

# type: ignore
from jose import jwt  # type:ignore

from fast_tmp.conf import settings

SECRET_KEY = settings.SECRET_KEY
ALGORITHM = settings.ALGORITHM


def create_access_token(data: dict, expires_delta: timedelta):
    to_encode = data.copy()
    expire = datetime.utcnow() + expires_delta
    to_encode.update({"exp": expire})
    encoded_jwt = jwt.encode(to_encode, SECRET_KEY, algorithm=ALGORITHM)
    return encoded_jwt


def decode_access_token(token: str):
    return jwt.decode(token, SECRET_KEY, algorithms=[ALGORITHM])



import datetime
from typing import Any, Dict, List, Optional, Tuple, Union

from pydantic import HttpUrl
from pydantic.main import BaseModel

# from fast_tmp.admin.schema.abstract_schema import Action
from fast_tmp.admin.schema.forms import Control  # , Limit
from fast_tmp.admin.schema.forms.enums import ControlEnum


class AddControl(BaseModel):
    type: str = "text"
    name: str
    label: str


class Item(BaseModel):
    label: str
    value: Union[str, int]


class ItemValidation(BaseModel):  # 验证工具
    pass


class ItemValidationError(BaseModel):
    pass


class NumberItem(Control):
    type: ControlEnum = ControlEnum.number
    min: Optional[int]
    max: Optional[int]
    precision: Optional[int]  # 小数点后几位
    step: Optional[int]  # 选择的步长
    value: Optional[int]
    showSteps: Optional[bool] = False


class NativeNumber(Control):
    type: ControlEnum = ControlEnum.native_number


class SelectOption(BaseModel):
    label: str
    value: Union[int, str]


class SelectItem(Control):
    type: ControlEnum = ControlEnum.select
    options: Optional[List[Union[SelectOption, str, int]]]
    source: Optional[str]  # 通过数据源里面获取，也可以配置地址从远程获取，值格式为:options:[{label:..,value:...,}]
    # children: Optional[List[Union[SelectOption, str, int]]]  # 这个在树结构在考虑
    multiple: bool = False  # 是否多选
    delimeter: Optional[str]
    labelField: Optional[str]
    valueField: Optional[str]
    joinValues: Optional[bool]
    extractValue: Optional[bool]
    checkAll: Optional[bool]  # 是否支持全选
    checkAllLabel: Optional[str]  # 全选的文字，默认为‘全选’
    defaultCheckAll: Optional[bool]  # 是否默认全选
    # 配置返回数组格式，具体参考https://baidu.gitee.io/amis/docs/components/form/
    # options#%E5%8A%A8%E6%80%81%E9%85%8D%E7%BD%AE
    searchable: bool = False  # 前端对选项是否启动搜索功能
    autoComplete: bool = True  # 是否对选项启动自动补全


class SelectItemCanModifyItem(SelectItem):
    """
    可以修改选项值的选择器
    """

    creatable: bool = False  # 是否支持新增选项
    addControls: Tuple[AddControl, ...] = (
        AddControl(type="text", name="label", label="选项标题"),
        AddControl(type="text", name="value", label="选项值"),
    )  # 配置弹框信息，第一个为标题，第二个为选项值
    addApi: Optional[HttpUrl]  # 配置增加接口，如果为空则不会保存
    editable: bool = False  # 前端是否可编辑
    editControls: Tuple[AddControl, ...] = (
        AddControl(type="text", name="label", label="选项标题"),
        AddControl(type="text", name="value", label="选项值"),
    )  # 配置修改值的弹框信息
    editApi: Optional[HttpUrl]
    deleteApi: Optional[HttpUrl]  # 配置删除接口


class ArrayItem(Control):
    type: ControlEnum = ControlEnum.array
    items: str = "text"  # 这个到时候改为枚举
    addable: bool = True  # 是否可新增
    removable: bool = True  # 是否可删除
    draggable: bool = False  # 是否可拖动排序，是否可以拖动排序, 需要注意的是当启用拖动排序的时候，
    # 会多一个$id 字段，具体请参考：https://baidu.gitee.io/amis/docs/components/form/array
    draggableTip: Optional[str]
    addButtonText: Optional[str]  # 新增按钮的文字
    minLength: Optional[int]  # 最短长度
    maxLength: Optional[int]  # 最长长度


class DatetimeItem(Control):
    type: ControlEnum = ControlEnum.datetime
    value: Optional[str]
    format: str = "YYYY-MM-DD HH:mm:ss"  # 'X'为时间戳格式,参考文档：
    # https://baidu.gitee.io/amis/zh-CN/docs/components/form/datetime
    inputFormat: str = "YYYY-MM-DD HH:mm:ss"  # 'X'为时间戳格式
    shortcuts: List[str] = []  # "yesterday" ,"today", "tomorrow",now,{n}hoursago : n 小时前，例
    # 如：1daysago，下面用法相同,{n}hourslater : n 小时前，例如：1daysago
    utc: bool = False
    clearable: bool = True
    embed: bool = False  # fixme:学习内联如何使用
    timeConstrainst: bool = True  # 不知道干吗用的


class DateItem(Control):
    type = ControlEnum.date
    value: Optional[str]
    format: str = "YYYY-MM-DD"  # 格式请参考文档：https://baidu.gitee.io/amis/zh-CN/docs/components/form/date  # 'X'为时间戳格式
    inputFormat: str = "YYYY-MM-DD"  # 'X'为时间戳格式
    shortcuts: List[str] = []  # "yesterday" ,"today", "tomorrow"
    utc: bool = False
    clearable: bool = True
    embed: bool = False  # fixme:学习内联如何使用
    timeConstrainst: bool = True  # 不知道干吗用的


class SwitchItem(Control):
    # Switch 开关
    type = ControlEnum.switch
    option: Optional[str]
    trueValue: Optional[int]
    falseValue: Optional[int]
    value: Optional[bool]


class RichTextItem(Control):
    # 目前富文本编辑器基于两个库：froala 和 tinymce，默认使用 tinymce。
    type = ControlEnum.rich_text
    body: Optional[Dict[str, Any]]


class TextItem(Control):
    type = ControlEnum.input_text
    body: Optional[Dict[str, Any]]
    trimContents: Optional[bool]  # 是否去除首尾空白文本。
    # clearable: Optional[bool]  # 是否可清除
    # resetValue: str = ""  # 清除后设置此配置项给定的值。


class TextareaItem(Control):
    # Textarea 多行文本输入框
    type = ControlEnum.textarea
    body: Optional[Dict[str, Any]]
    minRows: Optional[int]  # 最小行数
    maxRows: Optional[int]  # 最大行数
    trimContents: Optional[bool]  # 是否去除首尾空白文本。


class TimeItem(Control):
    # Time 时间
    type = ControlEnum.time
    body: Optional[Dict[str, Any]]
    value: Optional[datetime.time]  # 默认值
    timeFormat: str = "HH:mm"  # 时间选择器值格式，更多格式类型请参考 moment
    format: str = "X"  # 时间选择器值格式，更多格式类型请参考 moment
    inputFormat: str = "HH:mm"  # 时间选择器显示格式，即时间戳格式，更多格式类型请参考 moment
    placeholder: Optional[str]  # 占位文本
    clearable: bool = True  # 是否可清除
    timeConstrainst: Union[dict, bool]  # 请参考： react-datetime


class UUIDItem(Control):
    # 随机生成一个 id，可以用于防止表单重复提交。
    type = ControlEnum.uuid
    name: Optional[str]  # type: ignore
    length: Optional[int]


class CheckboxesItem(Control):
    """
    复选框
    """

    type = ControlEnum.checkboxes
    optional: Optional[Union[List[Dict[str, str]]]]  # 选项组
    source: str  # 动态选项组
    delimeter: bool = False  # 拼接符
    labelField: str = "label"  # 选项标签字段
    valueField: str = "value"  # 选项值字符
    joinValues: bool = False  # 拼接值
    extractValue: bool = True  # 提取值
    columnsCount: int = 1  # 选项按几列显示，默认为一列
    checkAll: bool = False  # 是否支持全选
    defaultCheckAll: bool = False  # 默认是否全选


class DynaticCheckboxesItem(CheckboxesItem):
    """
    可变值复选框
    """

    creatable: bool = False  # 新增选项
    createBtnLabel: Optional[str]  # 新增选项
    addControls: Optional[List[str]]  # 自定义新增单项
    addApi: Optional[HttpUrl]  # 配置新增选项接口
    editable: bool = False  # 编辑选项
    editControls: Optional[List[str]]  # 自定义编辑表单项
    editApi: Optional[HttpUrl]  # 配置编辑选项接口
    removable: bool = False  # 删除选项
    deleteApi: Optional[HttpUrl]  # 配置删除选项接口


class TransferItem(Control):
    # Transfer 穿梭器
    type = ControlEnum.transfer
    options: Optional[List[Union[dict, str]]]  # 选项组
    source: Optional[str]  # 动态选项组#fixme:路由替换需要支持source
    delimeter: Optional[str]  # 拼接符
    joinValues: Optional[bool]  # 拼接值
    extractValue: Optional[bool]  # 提取值
    labelField: Optional[str]  # 选项标签字段
    valueField: Optional[str]  # 选项值字段
    searchable: bool = False  # 当设置为 true 时表示可以通过输入部分内容检索出选项。
    searchApi: Optional[HttpUrl]  # 如果想通过接口检索，可以设置个 api。
    statistics: bool = True  # 是否显示统计数据
    selectTitle: Optional[str]  # 左侧的标题文字
    resultTitle: Optional[str]  # 右侧结果的标题文字
    sortable: bool = False  # 结果可以进行拖拽排序
    selectMode: str = "list"  # 可选：list、table、tree、chained、associated。分别为：列表形式、表格形式、树形选择形式、级联选择形式，关联选择形式（与级联选择的区别在于，级联是无限极，而关联只有一级，关联左边可以是个 tree）。
    searchResultMode: Optional[str]  # 如果不设置将采用 selectMode 的值，可以单独配置，参考 selectMode，决定搜索结果的展示形式。
    columns: Optional[List[dict]]  # 当展示形式为 table 可以用来配置展示哪些列，跟 table 中的 columns 配置相似，只是只有展示功能。
    leftOptions: Optional[List[dict]]  # 当展示形式为 associated 时用来配置左边的选项集。
    leftMode: Optional[str]  # 当展示形式为 associated 时用来配置左边的选择形式，支持 list 或者 tree。默认为 list。
    rightMode: Optional[str]  # 当展示形式为 associated 时用来配置右边的选择形式，可选：list、table、tree、chained。


class PickerItem(Control):
    # 列表选取，在功能上和 Select 类似，但它能显示更复杂的信息。默认和 Select 很像，但请看后面的 pickerSchema 设置。
    type = ControlEnum.picker
    options: Optional[List[Dict[str, str]]]  # 动态选项组
    source: Optional[str]
    multiple: bool = False
    delimeter: bool = False  # 拼接符
    labelField: str = "label"  # 选项标签字段
    valueField: str = "value"  # 选项值字符
    joinValues: bool = False  # 拼接值
    extractValue: bool = True  # 提取值
    autoFill: Optional[Dict[str, str]]  # 自动填充到某个位置
    modalMode: Optional[str]  # 配置弹出方式，默认为dialog，也可以配置drawer
    pickerSchema: Optional[Dict]  # 即用 List 类型的渲染，来展示列表信息。更多配置参考 CRUD
    embed: Optional[bool]  # 是否使用内嵌模式


# class ButtonToolbarItem(Control):
#     """
#     按钮组
#     """
#
#     type = "button-toolbar"
#     label: str = "按钮组"
#     buttons: List[_Action]
#
#
# class ChainSelectItem(Control):
#     # 链式下拉框
#     pass
#
#
# class CheckboxItem(Control):
#     # 勾选框
#     type = "checkbox"
#     option: Optional[str]  # 选项说明
#     trueValue: bool = True
#     falseValue: bool = False
#
#

#
# class CityItem(Control):
#     # 城市选择器
#     pass
#
#
# class ColorItem(Control):
#     # 颜色选择器
#     pass
#
#
# class ComboControlItem(Control):
#     # Combo组合中的表单项
#     type = "combo-control"
#     columnClassName: Optional[str]  # 列的类名，可以用它配置列宽度。默认平均分配。
#     unique: Optional[bool]  # 设置当前列值是否唯一，即不允许重复选择。
#
#
# class ComboItem(Control):
#     # Combo组合
#     type = "form"
#     debug: bool = True
#     api: Optional[HttpUrl]  #
#     mode: Optional[str]
#     multiLine: bool = False  # 是否分多行展示
#     multiple: bool = False  # 是否实现多选模式
#     minLength: Optional[int]  # 多选模式下配置Combo可添加的最少条数
#     maxLength: Optional[int]  # 多选模式下配置Combo可添加的最大条数
#     flat: bool = False  # 默认多选模式下，数据格式是对象数组的形式，当你配置的组合中只有一个表单项时，可以配置"flat": true，将值进行打平处理。
#     formClassName: Optional[str]  # 单组表单项的类名
#     controls: Optional[List[ComboControlItem]]  # 组合展示的表单项
#     joinValues: bool = True  # 默认为 true 当扁平化开启的时候，是否用分隔符的形式发送给后端，否则采用 array 的方式。
#     delimiter: str = "false"  # 当扁平化开启并且 joinValues 为 true 时，用什么分隔符。
#     addable: bool = False  # 是否可新增
#     removable: bool = False  # 是否可删除
#     deleteApi: Optional[HttpUrl]  # 如果配置了，则删除前会发送一个 api，请求成功才完成删除
#     deleteConfirmText: str = "确认要删除？"  # 当配置 deleteApi 才生效！删除时用来做用户确认
#     draggable: bool = False  # 是否可以拖动排序, 需要注意的是当启用拖动排序的时候，会多一个$id 字段
#     draggableTip: str = "可通过拖动每行中的【交换】按钮进行顺序调整"  # 可拖拽的提示文字
#     addButtonText: str = "新增"  # 新增按钮文字
#     scaffold: dict = {}  # 单组表单项初始值
#     canAccessSuperData: bool = False  # 指定是否可以自动获取上层的数据并映射到表单项上
#     conditions: Optional[dict]  # 数组的形式包含所有条件的渲染类型，单个数组内的test 为判断条件，数组内的controls为符合该条件后渲染的schema
#     typeSwitchable: bool = False  # 是否可切换条件，配合conditions使用
#     noBorder: bool = False  # 单组表单项是否显示边框
#     strictMode: bool = True  # 默认为严格模式，设置为 false 时，当其他表单项更新是，里面的表单项也可以及时获取，否则不会。
#     syncFields: Optional[str]  # 配置同步字段。只有 strictMode 为 false 时有效。如果 combo 层级比较深，
#     # 底层的获取外层的数据可能不同步。但是给 combo 配置这个属性就能同步下来。输入格式：["os"]
#
#
# class DiffEditorItem(Control):
#     # 对比编辑器
#     pass
#
#
# class EditorItem(Control):
#     # 代码编辑器
#     pass
#
#
# class FieldSetItem(Control):
#     # 表单项集合
#     type = "form"
#     api: Optional[HttpUrl]
#     className: Optional[str]  # CSS 类名
#     headingClassName: Optional[str]  # 标题 CSS 类名
#     bodyClassName: Optional[str]  # 内容区域 CSS 类名
#
#
# class FileItem(Control):
#     # 文件上传
#     type = "form"
#     reciever: Optional[HttpUrl]  # 上传文件接口
#     accept: str = "text/plain"  # 默认只支持纯文本，要支持其他类型，请配置此属性为文件后缀.xxx
#     asBase64: bool = False  # 将文件以base64的形式，赋值给当前组件
#     asBlob: bool = False  # 将文件以二进制的形式，赋值给当前组件
#     maxSize: Optional[int]  # 默认没有限制，当设置后，文件大小大于此值将不允许上传。单位为KB
#     maxLength: Optional[int]  # 默认没有限制，当设置后，一次只允许上传指定数量文件。
#     multiple: bool = False  # 是否多选。
#     joinValues: bool = True  # 拼接值
#     extractValue: bool = False  # 提取值
#     delimiter: str = ","  # 拼接符
#     autoUpload: bool = True  # 否选择完就自动开始上传
#     hideUploadButton: bool = False  # 隐藏上传按钮
#     stateTextMap: dict = {
#         "init": "",
#         "pending": "等待上传",
#         "uploading": "上传中",
#         "error": "上传出错",
#         "uploaded": "已上传",
#         "ready": "",
#     }  # 上传状态文案
#     fileField: str = "file"  # 如果你不想自己存储，则可以忽略此属性。
#     downloadUrl: Union[
#         str, bool
#     ] = ""  # 默认显示文件路径的时候会支持直接下载，可以支持加前缀如：http://xx.dom/filename= ，如果不希望这样，可以把当前配置项设置为 false。
#     useChunk: Union[
#         str, bool
#     ] = "auto"  # amis 所在服务器，限制了文件上传大小不得超出 10M，所以 amis 在用户选择大文件的时候，自动会改成分块上传模式。
#     chunkSize: int = 5 * 1024 * 1024  # 分块大小
#     startChunkApi: Optional[HttpUrl]  # startChunkApi
#     chunkApi: Optional[HttpUrl]  # chunkApi
#     finishChunkApi: Optional[HttpUrl]  # finishChunkApi
#
#
# class FormulaItem(Control):
#     # Formula公式
#     pass
#
#
# class GroupItem(Control):
#     # Group表单项组
#     pass
#
#
# class HBoxItem(Control):
#     # HBox
#     pass
#
#
# class HiddenItem(Control):
#     # Hidden隐藏字段
#     pass
#
#
# class ImageItem(Control):
#     # Image图片
#     type = "image"
#     reciever: Optional[HttpUrl]  # 上传文件接口
#     accept: str = ".jpeg,.jpg,.png,.gif"  # 支持的图片类型格式，请配置此属性为图片后缀，例如.jpg,.png
#     maxSize: Optional[int]  # 默认没有限制，当设置后，文件大小大于此值将不允许上传。单位为B
#     maxLength: Optional[int]  # 默认没有限制，当设置后，一次只允许上传指定数量文件。
#     multiple: bool = False  # 是否多选。
#     joinValues: bool = True  # 拼接值
#     extractValue: bool = False  # 提取值
#     delimiter: str = ","  # 拼接符
#     autoUpload: bool = True  # 否选择完就自动开始上传
#     hideUploadButton: bool = False  # 隐藏上传按钮
#     fileField: str = "file"  # 如果你不想自己存储，则可以忽略此属性。
#     crop: Union[bool, dict]  # 用来设置是否支持裁剪。
#     # limit:Optional[Limit] # 限制图片大小，超出不让上传。
#
#
# class InputGroupItem(Control):
#     # 输入框组合
#     pass
#
#
# class ListSchemaItem(Control):
#     # 列表一般用来实现选择，可以单选也可以多选，和 Radio/Checkboxs 最大的不同是在展现方面支持带图片。
#     type = "list"
#     clearable: bool = True
#     options: Union[List[dict], List[str]]
#     source: Union[HttpUrl, str]
#     multiple: bool = False
#     labelField: str = "label"
#     valueField: str = "value"
#     joinValues: bool = True
#     extractValue: bool = False
#     autoFill: Optional[dict]
#
#

#
#
# class MatrixItem(Control):
#     # 矩阵类型的输入框。
#     type = "page"
#
#
# class NestedSelectItem(Control):
#     # NestedSelect 级联选择器
#     type = "page"
#     body: Optional[dict]
#
#
# class PanelItem(Control):
#     # 展现上将多个 表单项 放同一个容器下。
#     type = "page"
#
#

#
#
# class RadiosItem(Control):
#     # 用于实现单选。
#     type = "page"
#     body: Optional[Dict[str, Any]]
#     options: Union[List[str], List[dict]]  # 选项组
#     source: Union[str, HttpUrl]  # 动态选项组
#     labelField: str = "label"  # 选项标签字段
#     valueField: str = "value"  # 选项值字段
#     columnsCount: int = 1  # 选项按几列显示，默认为一列
#     autoFill: Optional[dict]  # 自动填充
#
#
# class RatingItem(Control):
#     # Rating 评分
#     type = "page"
#
#
# class RangeItem(Control):
#     # Range 滑块,可以用于选择单个数值或数值范围。
#     type = "page"
#     body: Optional[Dict[str, Any]]
#
#
# class RepeatItem(Control):
#     # Repeat 重复频率选择器
#     type = "page"
#     body: Optional[Dict[str, Any]]
#
#

#
#
# class ServiceItem(Control):
#     # Service 功能容器
#     type = "page"
#     body: Optional[Dict[str, Any]]
#
#
# class SubFormItem(Control):
#     # SubForm 子表单
#     type = "page"
#     body: Optional[Dict[str, Any]]
#
#

#
#
# class StaticItem(Control):
#     # Static 静态展示   用来在表单中，展示静态数据
#     type = "page"
#     body: Optional[Dict[str, Any]]
#
#
# class TabsItem(Control):
#     # Tabs 选项卡   有多组输入框时，也可以通过选项卡来分组。
#     type = "page"
#     body: Optional[Dict[str, Any]]
#
#
# class TableItem(Control):
#     # Table 表格
#     type = "table"
#     body: Optional[Dict[str, Any]]
#     addable: bool = False  # 是否可增加一行
#     editable: bool = False  # 是否可编辑
#     removable: bool = False  # 是否可删除
#     showAddBtn: bool = True  # 是否显示添加按钮
#     addApi: Optional[HttpUrl]  # 新增时提交的 API
#     updateApi: Optional[HttpUrl]  # 修改时提交的 API
#     deleteApi: Optional[HttpUrl]  # 删除时提交的 API
#     addBtnLabel: Optional[str]  # 增加按钮名称
#     addBtnIcon: str = "fa fa-plus"  # 增加按钮图标
#     updateBtnLabel: str = ""  # 更新按钮名称
#     updateBtnIcon: str = "fa fa-pencil"  # 更新按钮图标
#     deleteBtnLabel: str = ""  # 删除按钮名称
#     deleteBtnIcon: str = "fa fa-minus"  # 删除按钮图标
#     confirmBtnLabel: str = ""  # 确认编辑按钮名称
#     confirmBtnIcon: str = "fa fa-check"  # 确认编辑按钮图标
#     cancelBtnLabel: str = ""  # 取消编辑按钮名称
#     cancelBtnIcon: str = "fa fa-times"  # 取消编辑按钮图标
#     needConfirm: bool = True  # 是否需要确认操作，，可用来控控制表格的操作交互
#     canAccessSuperData: bool = False  # 是否可以访问父级数据，也就是表单中的同级数据，通常需要跟 strictMode 搭配使用
#     strictMode: bool = True  # 为了性能，默认其他表单项项值变化不会让当前表格更新，有时候为了同步获取其他表单项字段，需要开启这个。
#     columns: Optional[List[Dict[str, bool]]]  # 列信息
#
#
# # class TagItem(AbstractControl):
# #     # Tag 标签选择器
# #     type = "page"
# #     body: Optional[Dict[str, Any]]
# #     options: List[Union[dict, str]]  # 选项组
# #     optionsTip: List[Union[dict, str]] = ["最近您使用的标签"]   # 选项提示
# #     source: str  # 动态选项组
# #     delimeter: str = 'false'  # 拼接符
# #     labelField: str = "label"    # 选项标签字段
# #     valueField: str = "value"    # 选项值字符
# #     joinValues: bool = True  # 拼接值
# #     extractValue: bool = False   # 提取值
# #     clearable: bool = False  # 在有值的时候是否显示一个删除图标在右侧。
# #     resetValue: str = ""  # 删除后设置此配置项给定的值。
#
#

#
#

#
#

#

#
# class TabsTransferItem(Control):
#     # TabsTransfer 组合穿梭器
#     pass
#
#
# class TreeItem(Control):
#     # Tree 树形选择框
#     type = "page"
#     body: Optional[Dict[str, Any]]
#     options: List[Union[dict, str]]  # 选项组
#     source: Union[str, HttpUrl]  # 动态选项组
#     autoComplete: bool = True  # 是否对选项启动自动补全
#     multiple: bool = False  # 是否多选
#     delimeter: Optional[str]  # 拼接符
#     labelField: str = "label"  # 选项标签字段
#     valueField: str = "value"  # 选项值字段
#     joinValues: bool = True  # 拼接值
#     extractValue: bool = False  # 提取值
#     creatable: bool = False  # 新增选项
#     addControls: Optional[List]  # 自定义新增表单项
#     addApi: Optional[HttpUrl]  # 配置新增选项接口
#     editable: bool = False  # 编辑选项
#     editControls: Optional[List]  # 自定义编辑表单项
#     editApi: Optional[HttpUrl]  # 配置编辑选项接口
#     removable: bool = False  # 删除选项
#     deleteApi: Optional[HttpUrl]  # 配置删除选项接口
#     searchable: bool = False  # 是否可检索，仅在 type 为 tree-select 的时候生效
#     hideRoot: bool = True  # 如果想要显示个顶级节点，请设置为 false
#     rootLabel: Union[bool, str] = "顶级"  # 当 hideRoot 不为 false 时有用，用来设置顶级节点的文字。
#     showIcon: bool = True  # 是否显示图标
#     showRadio: bool = False  # 是否显示单选按钮，multiple 为 false 是有效。
#     initiallyOpen: bool = True  # 设置是否默认展开所有层级。
#     unfoldedLevel: int = 0  # 设置默认展开的级数，只有initiallyOpen不是true时生效。
#     cascade: bool = False  # 当选中父节点时不自动选择子节点。
#     withChildren: bool = False  # 选中父节点时，值里面将包含子节点的值，否则只会保留父节点的值。
#     onlyChildren: bool = False  # 多选时，选中父节点时，是否只将其子节点加入到值中。
#     rootCreatable: bool = False  # 是否可以创建顶级节点
#     rootCreateTip: str = "添加一级节点"  # 创建顶级节点的悬浮提示
#     minLength: Optional[int]  # 最少选中的节点数
#     maxLength: Optional[int]  # 最多选中的节点数
#
#
# class TreeSelectItem(AbstractControl):
#     # TreeSelect 树形选择器
#     type = "page"
#     body: Optional[Dict[str, Any]]
#
# # fixme:type有问题
# # class YearItem(Control):
# #     # Year年
# #     type = "page"
# #     body: Optional[Dict[str, Any]]



from enum import Enum


class ButtonLevelEnum(str, Enum):
    """
    按钮的显示类型
    """

    info = "info"
    primary = "primary"
    secondary = "secondary"
    success = "success"
    warning = "warning"
    danger = "danger"
    light = "light"
    dark = "dark"
    link = "link"
    default = "default"


class Icon(str, Enum):
    """
    按钮的图标
    """

    pass


class ActionTypeEnum(str, Enum):
    """
    触发按钮的类型
    """

    dialog = "dialog"  # 弹框
    ajax = "ajax"  # ajax请求
    copy = "copy"  # 复制文本
    reload = "reload"  # 重载空间
    link = "link"  # 链接
    url = "url"
    drawer = "drawer"
    confirm = "confirm"
    cancel = "cancel"
    prev = "prev"
    next = "next"
    close = "close"


class TypeEnum(str, Enum):
    page = "page"
    crud = "crud"
    button = "button"
    action = "action"  # 行为按钮
    form = "form"
    operation = "operation"  # 这是啥？？
    hbox = "hbox"
    nav = "nav"
    app = "app"  # 多页应用


class ButtonSize(
    str,
    Enum,
):
    lg = "lg"  # 大
    md = "md"  # 中,默认值
    sm = "sm"  # 小
    xs = "xs"  # 极小


class DialogSize(str, Enum):
    full = "full"  # 全屏
    lg = "lg"  # 大
    md = "md"  # 中,默认值
    sm = "sm"  # 小
    xl = "xl"
    xs = "xs"  # 极小



from fastapi import HTTPException
from starlette import status

credentials_exception = HTTPException(  # 登录报错
    status_code=status.HTTP_401_UNAUTHORIZED,
    detail="未登录或权限过期",
    headers={"WWW-Authenticate": "Bearer"},
)
no_permission_exception = HTTPException(
    status_code=status.HTTP_403_FORBIDDEN,
    detail="权限不够",
    headers={"WWW-Authenticate": "Bearer"},
)
password_exception = HTTPException(  # 登录报错
    status_code=status.HTTP_400_BAD_REQUEST,
    detail="账户或密码错误",
)



from typing import List, Optional

from fastapi import APIRouter, Depends
from pydantic import BaseModel
from sqlalchemy import delete, func, select
from sqlalchemy.orm import Session
from starlette.requests import Request
from starlette.responses import RedirectResponse

from fast_tmp.site import ModelAdmin, get_model_site

from ..db import get_db_session
from ..models import User
from ..site.utils import clean_data_to_model, get_pk
from .depends import decode_access_token_from_data
from .responses import BaseRes, key_error, not_found_instance

router = APIRouter()


async def get_data(request: Request) -> dict:
    """
    从异步函数里面读取数据
    """
    return await request.json()


class ListDataWithPage(BaseModel):  # 带分页的数据
    items: List[dict]
    total: int = 0


@router.get("/{resource}/list")
def list_view(
    request: Request,
    page_model: ModelAdmin = Depends(get_model_site),
    perPage: int = 10,
    page: int = 1,
    session: Session = Depends(get_db_session),
    user: Optional[User] = Depends(decode_access_token_from_data),
):
    if not user:
        return RedirectResponse(request.url_for("admin:login"))

    datas = session.execute(
        select(page_model.list_display).limit(perPage).offset((page - 1) * perPage)
    )
    items = []
    for data in datas:
        items.append(dict(data))
    s = session.execute(select(func.count()).select_from(page_model.model))
    total = 0
    for i in s:
        total = i[0]
    return BaseRes(
        data=ListDataWithPage(
            total=total,
            items=items,
        )
    )


@router.put("/{resource}/update")
def update_data(
    request: Request,
    page_model: ModelAdmin = Depends(get_model_site),
    session: Session = Depends(get_db_session),
    data: dict = Depends(get_data),
    user: Optional[User] = Depends(decode_access_token_from_data),
):
    if not user:
        return RedirectResponse(request.url_for("admin:login"))
    data = clean_data_to_model(page_model.update_fields, data)
    w = get_pks(page_model, request)
    if isinstance(w, BaseRes):
        return w
    old_data = session.execute(select(page_model.model).where(*w)).scalar_one_or_none()
    if not old_data:
        return not_found_instance
    page_model.update_model(old_data, data)
    session.commit()
    return BaseRes()


@router.get("/{resource}/update")
def update_view(
    request: Request,
    page_model: ModelAdmin = Depends(get_model_site),
    session: Session = Depends(get_db_session),
    user: Optional[User] = Depends(decode_access_token_from_data),
):
    if not user:
        return RedirectResponse(request.url_for("admin:login"))

    w = get_pks(page_model, request)
    if isinstance(w, BaseRes):
        return w
    data = session.execute(select(page_model.update_fields).where(*w)).fetchone()
    if not data:
        return not_found_instance
    return BaseRes(data=dict(data))


@router.post("/{resource}/create")
def create(
    request: Request,
    data: dict = Depends(get_data),
    page_model: ModelAdmin = Depends(get_model_site),
    session: Session = Depends(get_db_session),
    user: Optional[User] = Depends(decode_access_token_from_data),
):
    if not user:
        return RedirectResponse(request.url_for("admin:login"))

    data = clean_data_to_model(page_model.create_fields, data)
    instance = page_model.create_model(data)
    session.add(instance)
    session.commit()
    return BaseRes(data=data)


@router.delete("/{resource}/delete")
def delete_one(
    request: Request,
    page_model: ModelAdmin = Depends(get_model_site),
    session: Session = Depends(get_db_session),
    user: Optional[User] = Depends(decode_access_token_from_data),
):
    if not user:
        return RedirectResponse(request.url_for("admin:login"))

    w = get_pks(page_model, request)
    if isinstance(w, BaseRes):
        return w
    session.execute(delete(page_model.model).where(*w))
    session.commit()
    return BaseRes()


def get_pks(page_model: ModelAdmin, request: Request):
    params = dict(request.query_params)
    pks = get_pk(page_model.model)
    w = []
    for k, v in params.items():
        if pks.get(k) is not None:
            w.append(pks[k] == v)
        else:
            return key_error
    return w


class DIDS(BaseModel):
    ids: List[int]


#  todo next version
# @router.post("/{resource}/deleteMany/")
# def bulk_delete(request: Request, ids: DIDS,
#                 model_site: Optional[ModelAdmin] = Depends(get_model_site),
#
#                 ):
#     # await model.filter(pk__in=ids.ids).delete()
#     return BaseRes()


@router.get("/{resource}/schema")
def get_schema(
    request: Request,
    page: ModelAdmin = Depends(get_model_site),
    user: Optional[User] = Depends(decode_access_token_from_data),
):
    if not user:
        return RedirectResponse(request.url_for("admin:login"))

    return BaseRes(data=page.get_app_page())



from typing import Dict, List, Optional, Union

from pydantic import BaseModel

from fast_tmp.admin.schema.abstract_schema import BaseAmisModel
from fast_tmp.admin.schema.enums import TypeEnum
from fast_tmp.admin.schema.forms.enums import ControlEnum, FormWidgetSize, ItemModel


# fixme:未来考虑更多的fields类型字段支持
class Column(BaseModel):
    """
    用于列表等的显示
    """

    type: ControlEnum = ControlEnum.text  # 把这个和schema获取的参数进行融合，保证schema获取的值可以使用
    name: str  # type: ignore
    label: str


class Mapping(Column):
    """
    专门用来作为枚举显示用的
    """

    type = ControlEnum.mapping
    map: Dict[Union[int, str], str]  # map的值可以是html片段


class AbstractControl(Column):
    pass


class Form(BaseAmisModel):
    type = TypeEnum.form
    name: str
    title: Optional[str]
    submitText: Optional[str]
    wrapWithPanel: Optional[bool]
    api: str
    initApi: Optional[str]
    # interval: int = 3000??
    primaryField: Optional[str]  # 设置主键"id"
    body: List[AbstractControl]


class Control(AbstractControl):
    """
    用户form表单等写入
    """

    labelRemark: Optional[str]  # 表单项标签描述
    description: Optional[str]  # 描述
    placeholder: Optional[str]  # 描述
    inline: bool = False  # 内联样式
    submitOnChange: bool = False  # 是否该表单项值发生变化时就提交当前表单。
    disabled: bool = False
    disableOn: Optional[str]  # 配置规则查看https://baidu.gitee.io/amis/docs/components/form/formitem
    visible: Optional[bool]
    visibleOn: Optional[str]  # 配置规则查看https://baidu.gitee.io/amis/docs/components/form/formitem
    required: bool = True
    requiredOn: Optional[str]
    hidden: Optional[bool]  # 可使用条件配置如 this.number>1
    hiddenOn: Optional[str]  # 配置判定逻辑
    validations: Optional[Dict[str, Union[int, str]]]  # 注意，键值对请参考ValidateEnum
    validationErrors: Optional[
        Dict[str, str]
    ]  # 注意，键值对请参考ValidateEnum，举例："minimum": "同学，最少输入$1以上的数字哈",其中$1为该错误的数据
    className: Optional[str]  # 表单最外层类名
    inputClassName: Optional[str]  # 表单控制器类名
    labelClassName: Optional[str]  # label 的类名
    mode: ItemModel = ItemModel.normal
    size: FormWidgetSize = FormWidgetSize.md



from typing import Any

from fast_tmp.models import User
from fast_tmp.site import ModelAdmin


class UserAdmin(ModelAdmin):
    model = User
    list_display = (User.id, User.username, User.is_active)
    create_fields = (User.username, User.password)
    update_fields = (User.password,)

    @classmethod
    def create_model(cls, data: dict) -> Any:
        user = cls.model(**data)
        user.set_password(data["password"])
        return user

    @classmethod
    def update_model(cls, model: User, data: dict) -> Any:
        model.set_password(data["password"])
        return model



import jinja2
from pydantic import typing


def register_tags(templates):
    env = templates.env

    @jinja2.pass_context
    def static(context: dict, **path_params: typing.Any) -> str:
        request = context["request"]
        path = "admin"
        return request.url_for(path, **path_params)

    env.globals["static"] = static



# 弹框
from typing import List, Optional, Union

from pydantic.main import BaseModel

from .abstract_schema import _Action
from .enums import DialogSize


class Dialog(BaseModel):
    title: str
    nextCondition: bool = True
    size: DialogSize = DialogSize.md
    actions: Optional[List[_Action]]
    body: Union[str, BaseModel]


# todo:尚未完成：https://baidu.gitee.io/amis/docs/components/dialog?page=1

# 抽屉
class Drawer(BaseModel):
    title: str
    body: Union[str, BaseModel]
    actions: Optional[Union[_Action, List[_Action]]] = None



from typing import List, Optional, Tuple, Union

from fast_tmp.admin.schema.abstract_schema import BaseAmisModel

from .enums import TypeEnum


class Page(BaseAmisModel):
    type = TypeEnum.page
    title: Optional[str]
    subTitle: Optional[str]
    remark: Optional[str]
    aside: Optional[Union[BaseAmisModel, List[BaseAmisModel]]]  # 往页面的边栏区域加内容
    toolbar: Optional[
        Union[BaseAmisModel, List[BaseAmisModel]]
    ]  # 往页面的右上角加内容，需要注意的是，当有 title 时，该区域在右上角，没有时该区域在顶部
    body: List[BaseAmisModel] = []
    initApi: Optional[str]  # 获取初始数据
    initFetch: bool = False  # 是否进行初始数据获取

    _list_fields: Tuple[str, ...] = ()
    _update_fields: Tuple[str, ...] = ()
    _create_fields: Tuple[str, ...] = ()


class HBox(BaseAmisModel):
    type = TypeEnum.hbox
    className: Optional[str]
    columns: List[BaseAmisModel]


#
# class AppPage(BaseModel):
#     """
#     app的子页
#     """
#
#     label: str
#     icon: str
#     url: str
#     schema_: Page = Field(..., alias="schema")
#     # schemaApi: Optional[str] = None
#     link: Optional[str] = None
#     redirect: Optional[str] = None
#     rewrite: Optional[str] = None
#     isDefaultPage: Optional[str] = None
#
#     # visible:Optional[bool]=None
#     def dict(self, *args, **kwargs):
#         kwargs["exclude_none"] = True
#         res: dict = super().dict(*args, **kwargs)
#         return res


# class AppPageGroup(BaseModel):
#     label: str
#     children: List[AppPage]


# class App(BaseAmisModel):
#     type = TypeEnum.app
#     api: Optional[str]  # 页面配置接口，如果你想远程拉取页面配置请配置。返回配置路径 json>data>pages，具体格式请参考 pages 属性。
#     brandName: str  # 应用名称
#     logo: Optional[str]  # 图片地址或svg
#     header: Optional[BaseModel]  # 顶部区域
#     asideBefore: Optional[BaseModel]  # 页面菜单上前面区域
#     asideAfter: Optional[BaseModel]  # 页面菜单下前面区域
#     footer: Optional[BaseModel]  # 页面
#     pages: List[AppPageGroup]  # 具体页面数组，地一层为label集合，真正的页面在第二层开始



import os
import dotenv
dotenv.load_dotenv()

# import sentry_sdk
# from sentry_sdk.integrations.redis import RedisIntegration

BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))

DEBUG = os.getenv("DEBUG") == "True"

TORTOISE_ORM = {
    'connections': {
        'default': {
            'engine': 'tortoise.backends.mysql',
            'credentials': {
                'host': os.getenv("DB_HOST"),
                'port': os.getenv("DB_PORT"),
                'user': os.getenv("DB_USER"),
                'password': os.getenv("DB_PASSWORD"),
                'database': os.getenv("DB_NAME"),
            }
        },
    },
    'apps': {
        'fast_tmp': {
            'models': ['fast_tmp.models', 'aerich.models'],  # 注册app.models
            'default_connection': 'default',
        }
    }
}
EXTRA_SCRIPT=[] # 自定义执行脚本
# redis配置
# REDIS_HOST = os.getenv("REDIS_HOST", "127.0.0.1")
# REDIS_PORT = os.getenv("REDIS_PORT", 6379)
# REDIS_PASSWORD = os.getenv("REDIS_PASSWORD")
# REDIS = {
#     "host": REDIS_HOST,
#     "port": REDIS_PORT,
#     "password": REDIS_PASSWORD,
#     "db": 2,
#     "encoding": "utf-8",
# }
# sentry配置
# sentry_sdk.init(
#     dsn=os.getenv("SENTRY_DSN"),
#     environment=os.getenv("ENVIRONMENT", "development"),
#     integrations=[RedisIntegration()],
# )
# logging
# LOGGER = logging.getLogger("example")
# if DEBUG:
#     LOGGER.setLevel(logging.DEBUG)
# else:
#     LOGGER.setLevel(logging.INFO)
# sh = logging.StreamHandler(sys.stdout)
# sh.setLevel(logging.DEBUG)
# sh.setFormatter(
#     logging.Formatter(
#         fmt="%(asctime)s - %(name)s:%(lineno)d - %(levelname)s - %(message)s",
#         datefmt="%Y-%m-%d %H:%M:%S",
#     )
# )
# LOGGER.addHandler(sh)



from typing import Any, Dict, List, Optional, Tuple, Type

from pydantic import BaseModel
from sqlalchemy import Column

from fast_tmp.admin.schema.actions import AjaxAction, DialogAction
from fast_tmp.admin.schema.buttons import Operation
from fast_tmp.admin.schema.crud import CRUD
from fast_tmp.admin.schema.enums import ButtonLevelEnum
from fast_tmp.admin.schema.forms import Form
from fast_tmp.admin.schema.frame import Dialog
from fast_tmp.admin.schema.page import Page

from ..admin.constant import crud_root_rooter
from .utils import get_columns_from_model, get_controls_from_model, get_pk


class ModelAdmin:
    model: Any  # model
    __name: Optional[str] = None
    # list
    list_display: Tuple[Column, ...] = ()
    # list_display_links: Tuple[Column, ...] = ()
    list_per_page: int = 10
    list_max_show_all: int = 200
    # search list
    search_fields: Tuple[Column, ...] = ()
    filter_fields: Tuple[Column, ...] = ()
    # create
    create_fields: Tuple[Column, ...] = ()
    # exclude: Tuple[Union[str, BaseModel], ...] = ()
    # update ,如果为空则取create_fields
    update_fields: Tuple[Column, ...] = ()

    methods: Tuple[str, ...] = ("list", "retrieve", "create", "put", "delete", "deleteMany")
    # actions_func: Dict[
    #     str, Callable
    # ] = (
    #     {}
    # )  # 请求对应的调用函数，"list","retrieve","create","put","delete","deleteMany"有默认自定义方法，这里如果填了就是覆盖默认方法
    # 页面相关的东西
    __create_dialog: Any = None
    __get_pks: Any = None
    page_model: Type[BaseModel]
    prefix: str = "/admin"

    @classmethod
    def name(cls):
        if cls.__name is None:
            cls.__name = cls.model.__name__
        return cls.__name

    @classmethod
    def get_create_dialogation_button(cls):
        if cls.__create_dialog is None:
            cls.__create_dialog = DialogAction(
                label="新增",
                dialog=Dialog(
                    title="新增",
                    body=Form(
                        name=f"新增{cls.name()}",
                        title=f"新增{cls.name()}",
                        body=get_controls_from_model(cls.create_fields),
                        api=f"post:{crud_root_rooter}{cls.name()}/create",
                    ),
                ),
            )
        return cls.__create_dialog

    @classmethod
    def get_list_page(cls):
        return get_columns_from_model(cls.list_display)

    @classmethod
    def pks(cls):
        if cls.__get_pks is None:
            primary_keys = get_pk(cls.model).keys()
            del_path = []
            for pk in primary_keys:
                del_path.append(f"{pk}=${pk}")
            cls.__get_pks = "".join(del_path)
        return cls.__get_pks

    @classmethod
    def get_del_one_button(cls):
        return AjaxAction(
            label="删除",
            type="button",
            level=ButtonLevelEnum.danger,
            confirmText="确认要删除？",
            api="delete:" + crud_root_rooter + cls.name() + "/delete?" + cls.pks(),
        )

    @classmethod
    def get_update_one_button(cls):
        return DialogAction(
            label="修改",
            dialog=Dialog(
                title="修改",
                body=Form(
                    name=f"修改{cls.name()}",
                    body=get_controls_from_model(cls.update_fields),
                    api="put:" + crud_root_rooter + cls.name() + "/update?" + cls.pks(),
                    initApi="get:" + crud_root_rooter + cls.name() + "/update?" + cls.pks(),
                ),
            ),
        )

    @classmethod
    def get_operation(cls):
        primary_keys = get_pk(cls.model).keys()
        del_path = []
        for pk in primary_keys:
            del_path.append(f"{pk}=${pk}")
        return Operation(buttons=[cls.get_del_one_button(), cls.get_update_one_button()])

    @classmethod
    def get_crud(cls):
        body = []
        body.append(cls.get_create_dialogation_button())
        columns = []
        columns.extend(cls.get_list_page())
        columns.append(cls.get_operation())
        body.append(
            CRUD(
                api=crud_root_rooter + cls.name() + "/list",
                columns=columns,
            )
        )
        return body

    @classmethod
    def get_app_page(cls):
        return Page(title=cls.name(), body=cls.get_crud()).dict(exclude_none=True)

    @classmethod
    def create_model(cls, data: dict):
        """
        写入数据库之前调用
        """
        return cls.model(**data)

    @classmethod
    def update_model(cls, model: Any, data: dict) -> Any:
        """
        更新数据之前调用
        """
        for k, v in data.items():
            setattr(model, k, v)
        return model


model_list: Dict[str, List[Type[ModelAdmin]]] = {}


def register_model_site(model_group: Dict[str, List[Type[ModelAdmin]]]):
    model_list.update(model_group)


def get_model_site(resource: str) -> Optional[Type[ModelAdmin]]:
    for m_l in model_list.values():
        for i in m_l:
            if i.name() == resource:
                return i
    return None



import os

os.environ.setdefault("FASTAPI_SETTINGS_MODULE", '{{cookiecutter.project_slug}}.settings')
from {{cookiecutter.project_slug}}.factory import create_app

app = create_app()

if __name__ == '__main__':
    import uvicorn

    uvicorn.run(app, debug=True, port=8000, lifespan="on")



from typing import Any

from pydantic import BaseModel


class BaseRes(BaseModel):
    status: int = 0
    msg: str = ""
    data: Any = {}


key_error = BaseRes(status=400, msg="主键错误")
not_found_instance = BaseRes(status=404, msg="找不到对象")



from datetime import datetime
from typing import Dict, List, Optional, Text, Tuple

from sqlalchemy import (
    BOOLEAN,
    DECIMAL,
    INTEGER,
    BigInteger,
    Boolean,
    Column,
    DateTime,
    Enum,
    Float,
    Integer,
    Numeric,
    SmallInteger,
    inspect,
)
from sqlalchemy.orm import Mapper

from fast_tmp.admin.schema.forms import Column as AmisColumn
from fast_tmp.admin.schema.forms import Column as FormColumn
from fast_tmp.admin.schema.forms import Control, FormWidgetSize, ItemModel
from fast_tmp.admin.schema.forms.widgets import (
    DatetimeItem,
    NativeNumber,
    NumberItem,
    SwitchItem,
    TextareaItem,
    TextItem,
)


def get_pk(model) -> Dict[str, Column]:
    mapper: Mapper = inspect(model)
    primary_keys: Tuple[Column, ...] = mapper.primary_key
    return {pk.name: pk for pk in primary_keys}


def _get_base_attr(field_type: Column, **kwargs) -> dict:
    res = dict(
        className=None,
        inputClassName=None,
        labelClassName=None,
        name=field_type.name,
        label=field_type.name,
        labelRemark=None,
        description=None,
        placeholder=None,
        inline=False,
        submitOnChange=False,
        disabled=False,
        disableOn=None,
        required=not field_type.nullable,
        mode=ItemModel.normal,
        size=FormWidgetSize.full,
    )
    if field_type.default is not None:

        if callable(field_type.default):
            res["value"] = field_type.default()
        else:
            res["value"] = field_type.default.arg
    res.update(kwargs)
    return res


def get_controls_from_model(include: Tuple[Column, ...] = ()) -> List[Control]:
    """
    从pydantic_queryset_creator创建的schema获取字段
    extra_fields:额外的自定义字段
    """
    res: List[Control] = []
    for field in include:
        item: Optional[Control] = None
        if isinstance(field.type, Enum):
            pass
        elif isinstance(field.type, (Boolean, BOOLEAN)):  # boolean's default value must be false.
            item = SwitchItem(
                **_get_base_attr(field),
            )
        elif isinstance(field.type, Float):
            item = NumberItem(
                **_get_base_attr(field),
            )
        elif isinstance(field.type, (DECIMAL, Numeric)):
            item = NativeNumber(
                **_get_base_attr(field),
            )
            # todo need add limit check
        # elif isinstance(field.type,Time):
        #     item = TimeItem(
        #         **_get_base_attr(field),
        #
        #     )
        elif isinstance(field.type, DateTime):
            item = DatetimeItem(  # fixme need check
                **_get_base_attr(field), format="YYYY-MM-DDTHH:mm:ss"
            )
        # elif isinstance(field.type, BigInteger):
        #     pass
        # elif isinstance(field.type, SmallInteger):
        #     pass
        elif isinstance(field.type, (Integer, INTEGER, BigInteger, SmallInteger)):
            # min = (
            #     field.type.constraints.get("ge")  # type :ignore
            #     if hasattr(field.type, "constraints")  # type :ignore
            #     else -2147483648  # type :ignore
            # )  # type :ignore
            # max = (  # type :ignore
            #     field.type.constraints.get("le")  # type :ignore
            #     if hasattr(field.type, "constraints")  # type :ignore
            #     else 2147483647  # type :ignore
            # )  # type :ignore
            item = NumberItem(
                # min=min,
                # max=max,
                precision=0,
                step=1,
                **_get_base_attr(field),
                # validations={
                #     "minimum": min,
                #     "maximum": max,
                # },
            )
        elif isinstance(field.type, Text):
            item = TextareaItem(
                **_get_base_attr(field),
            )
        else:
            item = TextItem(
                **_get_base_attr(field),
            )
            if hasattr(field.type, "length"):
                validations = {
                    "maxLength": field.type.length,
                }
                item.validations = validations
        if item is not None:
            res.append(item)
    return res


def get_columns_from_model(
    include: Tuple[Column, ...] = (),
) -> List[AmisColumn]:
    """
    从pydantic_queryset_creator创建的schema获取字段
    todo：增加多对多字段显示
    """
    res = []
    for field in include:
        # if isinstance(field_type, (IntEnumFieldInstance, CharEnumFieldInstance)):
        #     res.append(
        #         Mapping(
        #             name=field,
        #             label=field,
        #             map={k: v for k, v in field_type.enum_type.choices.items()},
        #         )
        #     )
        #     fixme:处理开关字段
        # elif isinstance(field_type,):#fixme:处理特殊字段，比如json字段，或者图片、开关等需要特殊显示的类型。
        # elif isinstance(field_type, ManyToManyFieldInstance):
        #     continue
        # else:
        res.append(FormColumn(name=field.name, label=field.name))
    return res


def clean_data_to_model(model_field: Tuple[Column, ...], data: dict) -> dict:
    """
    把前端的数据转为后端的格式
    """
    for k in data:
        for i in model_field:
            if i.name == k:
                if isinstance(i.type, DateTime):
                    data[k] = datetime.strptime(data[k], "%Y-%m-%dT%H:%M:%S")
                break
        else:
            del data[k]
    return data



import os
from datetime import timedelta
from typing import Optional

from fastapi import Depends, FastAPI, Form, Request
from fastapi.security import OAuth2PasswordBearer
from fastapi.templating import Jinja2Templates
from sqlalchemy.orm import Session
from starlette.responses import RedirectResponse
from starlette.staticfiles import StaticFiles

from fast_tmp.admin.depends import authenticate_user, decode_access_token_from_data
from fast_tmp.conf import settings
from fast_tmp.db import get_db_session
from fast_tmp.models import User
from fast_tmp.models.site import UserAdmin
from fast_tmp.site import model_list, register_model_site
from fast_tmp.utils.token import create_access_token

from ..jinja_extension.tags import register_tags
from .constant import crud_root_rooter, model_router
from .endpoint import router
from .responses import BaseRes

oauth2_scheme = OAuth2PasswordBearer(tokenUrl=settings.LOGIN_URL)
base_path = os.path.dirname(__file__)
templates = Jinja2Templates(directory=base_path + "/templates")
register_tags(templates)
admin = FastAPI(title="fast-tmp")
# todo add debug,
admin.mount("/static", app=StaticFiles(directory=base_path + "/static"), name="static")

register_model_site({"Auth": [UserAdmin]})
admin.include_router(router, prefix=model_router)


@admin.get(
    "/",
    name="index",
)
def index(request: Request, user: Optional[User] = Depends(decode_access_token_from_data)):
    if not user:
        return RedirectResponse(request.url_for("admin:login"))
    return templates.TemplateResponse(
        "index.html",
        {"request": request, "title": admin.title},
    )


@admin.post(
    "/",
    name="index",
)  # bug:can not use admin.route .It can't check depends.
def index_post(request: Request, user: Optional[User] = Depends(decode_access_token_from_data)):
    if not user:
        return RedirectResponse(request.url_for("admin:login"))

    return templates.TemplateResponse(
        "index.html",
        {"request": request, "title": admin.title},
    )


@admin.post("/login")
def login(
    request: Request,
    username: Optional[str] = Form(None),
    password: Optional[str] = Form(None),
    session: Session = Depends(get_db_session),
):
    context = {
        "request": request,
        "errinfo": "",
        "username_err": False,
        "password_err": False,
    }
    if not username:
        context["username_err"] = True
        return templates.TemplateResponse("login.html", context)
    if not password:
        context["password_err"] = True
        return templates.TemplateResponse("login.html", context)

    user = authenticate_user(username, password, session)
    if not user:
        context["errinfo"] = "e"
        return templates.TemplateResponse("login.html", context)

    access_token_expires = timedelta(minutes=settings.ACCESS_TOKEN_EXPIRE_MINUTES)
    access_token = create_access_token(
        data={"sub": user.username, "id": user.id}, expires_delta=access_token_expires
    )
    res = RedirectResponse(
        request.url_for(
            "admin:index",
        ),
    )
    res.set_cookie("access_token", access_token, expires=settings.ACCESS_TOKEN_EXPIRE_MINUTES * 60)
    return res


@admin.get("/login", name="login")
def login_html(request: Request):
    context = {
        "request": request,
        "errinfo": "",
        "username_err": False,
        "password_err": False,
    }
    return templates.TemplateResponse("login.html", context)


@admin.get("/site")
def get_site(request: Request, user: Optional[User] = Depends(decode_access_token_from_data)):
    if not user:
        return RedirectResponse(request.url_for("admin:login"))
    pages = []
    for name, ml in model_list.items():  # todo add home page
        pages.append(
            {
                "label": name,
                "children": [
                    {
                        "label": model.name(),
                        "url": model.name(),
                        "schemaApi": crud_root_rooter + model.name() + "/schema",
                    }
                    for model in ml
                ],
            }
        )
    return BaseRes(data={"pages": pages})



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['fast_tmp',
 'fast_tmp.admin',
 'fast_tmp.admin.schema',
 'fast_tmp.admin.schema.forms',
 'fast_tmp.conf',
 'fast_tmp.jinja_extension',
 'fast_tmp.models',
 'fast_tmp.site',
 'fast_tmp.utils',
 'fastapi_cli',
 'fastapi_cli.tpl.app.{{cookiecutter.app_name}}',
 'fastapi_cli.tpl.app.{{cookiecutter.app_name}}.routes',
 'fastapi_cli.tpl.project.{{cookiecutter.project_slug}}',
 'fastapi_cli.tpl.project.{{cookiecutter.project_slug}}.tests',
 'fastapi_cli.tpl.project.{{cookiecutter.project_slug}}.{{cookiecutter.project_slug}}',
 'fastapi_cli.tpl.project.{{cookiecutter.project_slug}}.{{cookiecutter.project_slug}}.apps',
 'fastapi_cli.tpl.project.{{cookiecutter.project_slug}}.{{cookiecutter.project_slug}}.apps.api',
 'fastapi_cli.tpl.project.{{cookiecutter.project_slug}}.{{cookiecutter.project_slug}}.apps.api.v1',
 'fastapi_cli.tpl.project.{{cookiecutter.project_slug}}.{{cookiecutter.project_slug}}.apps.api.v1.endpoints',
 'fastapi_cli.tpl.project.{{cookiecutter.project_slug}}.{{cookiecutter.project_slug}}.enums']

package_data = \
{'': ['*'],
 'fast_tmp.admin': ['static/amis/*',
                    'static/css/*',
                    'static/img/*',
                    'static/js/*',
                    'templates/*'],
 'fastapi_cli': ['tpl/app/*',
                 'tpl/project/*',
                 'tpl/static/*',
                 'tpl/static/static/*'],
 'fastapi_cli.tpl.project.{{cookiecutter.project_slug}}': ['templates/*']}

install_requires = \
['typer>=0.4.0,<0.5.0']

entry_points = \
{'console_scripts': ['fast-tmp = fastapi_cli:main']}

setup_kwargs = {
    'name': 'fast-tmp',
    'version': '0.9.0',
    'description': '',
    'long_description': None,
    'author': 'Chise1',
    'author_email': 'chise123@live.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/Chise1/fast-tmp',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



from typing import Any, List

from sqlalchemy import Boolean, Column, ForeignKey, Integer, String, Table, select
from sqlalchemy.orm import Session, joinedload, relationship
from sqlalchemy.orm.decl_api import declarative_base

from fast_tmp.utils.password import make_password, verify_password

Base: Any = declarative_base()

group_permission = Table(
    "auth_group_permission",
    Base.metadata,
    Column("group_id", Integer, ForeignKey("auth_group.id")),
    Column("permission_code", String(128), ForeignKey("auth_permission.code")),
)

group_user = Table(
    "auth_group_user",
    Base.metadata,
    Column("group_id", Integer, ForeignKey("auth_group.id")),
    Column("auth_user_id", Integer, ForeignKey("auth_user.id")),
)


class AbstractModel(Base):
    __abstract__ = True
    id = Column(Integer, primary_key=True)


class User(AbstractModel):
    __tablename__ = "auth_user"

    username = Column(String(128), unique=True)
    password = Column(String(128), nullable=True)
    is_superuser = Column(Boolean(), default=False)
    is_manager = Column(Boolean(), default=False)  # could login admin
    is_active = Column(Boolean(), default=True)

    def set_password(self, raw_password: str):
        """
        设置密码
        :param raw_password:
        :return:
        """
        self.password = make_password(raw_password)

    def verify_password(self, raw_password: str) -> bool:
        """
        验证密码
        """
        return verify_password(raw_password, self.password)

    def has_perm(
        self,
        perm: str,
        session: Session,
    ) -> bool:
        """
        判定用户是否有权限
        """
        if self.is_superuser:
            return True
        results = session.execute(
            select(Group.id)
            .join(Group.users.and_(User.id == self.id))
            .join(Group.permissions.and_(Permission.code == perm))
        ).all()
        if len(results) > 0:
            return True
        return False

    # todo:need test
    # def has_perms(self, session: Session, perms: Set[str]) -> bool:
    #     """
    #     根据permission的codename进行判定
    #     """
    #     if self.is_superuser:
    #         return True
    #     results = session.execute(
    #         select(Permission.code)
    #         .join(Group.permissions.and_(Permission.code.in_(perms)))
    #         .join(Group.users.and_(User.id == self.id))
    #     )
    #     # results = session.execute(
    #     #     select(Group.id)
    #     #         .join(Group.users.and_(User.id == self.id))
    #     #         .join(Group.permissions.and_(Permission.code.in_(perms)))
    #     # ).all()
    #     if len(results) == len(perms):
    #         return True
    #     return False

    def perms(self, session: Session) -> List[str]:
        """
        获取用户的所有权限
        """
        groups = (
            session.execute(
                select(Group)
                .options(joinedload(Group.permissions))
                .join(Group.users)
                .filter(User.id == self.id)
            )
            .scalars()
            .all()
        )
        permissions = []
        for group in groups:
            for p in group.permissions:
                permissions.append(p.code)
        return permissions

    def __str__(self):
        return self.username


class Permission(Base):
    __tablename__ = "auth_permission"
    code = Column(String(128), primary_key=True)
    name = Column(String(128))

    def __str__(self):
        return self.name + "-" + self.code


class Group(AbstractModel):
    __tablename__ = "auth_group"
    name = Column(String(32))
    permissions: List[Permission] = relationship(
        "Permission", secondary="auth_group_permission", backref="groups", cascade="all,delete"
    )
    users: List[User] = relationship(
        "User", secondary="auth_group_user", backref="groups", cascade="all,delete"
    )

    def get_perms(self, db_session: Session) -> List[str]:
        permissions = db_session.execute(
            select(group_permission).where(group_permission.c.group_id == self.id)
        ).fetchall()
        return [permission[1] for permission in permissions]

    # def get_users(self, session: Session) -> List[User]:
    #     results = session.execute(
    #         select(Group).options(joinedload(Group.users)).where(Group.id == self.id)
    #     )
    #     return results.scalars().first().users



from pydantic.main import BaseModel
from pydantic.networks import HttpUrl

from fast_tmp.admin.schema.abstract_schema import BaseAmisModel

from .enums import TypeEnum


class NavLinks(BaseModel):
    label: str
    to: HttpUrl


class Nav(BaseAmisModel):
    type = TypeEnum.nav
    stacked: bool = True
    className: str = "w-md"



from typing import Optional

from fastapi import Cookie, Depends
from sqlalchemy import select
from sqlalchemy.orm import Session

from fast_tmp.db import get_db_session
from fast_tmp.models import User
from fast_tmp.utils.token import decode_access_token


def decode_access_token_from_data(
    access_token: Optional[str] = Cookie(None), session: Session = Depends(get_db_session)
) -> Optional[User]:
    if access_token is not None:
        try:
            payload = decode_access_token(access_token)
            username: str = payload.get("sub")
            if username:
                user = get_user(username, session)
                if user:
                    return user
        except Exception:  # nosec
            pass
    return None


def get_user(username: str, session: Session) -> Optional[User]:
    res = session.execute(
        select(User)  # type: ignore
        .where(User.username == username)
        .where(User.is_active == True)  # noqa
    )

    return res.scalar_one_or_none()  # type: ignore


def authenticate_user(username: str, password: str, session: Session) -> Optional[User]:
    """
    验证密码
    """
    user = get_user(username, session)
    if user and user.verify_password(password):
        return user
    return None



# -*- encoding: utf-8 -*-
"""
@File    : css.py
@Time    : 2021/1/1 14:08
@Author  : chise
@Email   : chise123@live.com
@Software: PyCharm
@info    :
"""
from pydantic import BaseModel


class Horizontal(BaseModel):
    left: int
    right: int
    offset: int
